from dataclasses import dataclass
from datetime import datetime

from dynatrace import Dynatrace
from dynatrace.environment_v1.synthetic_third_party import SYNTHETIC_EVENT_TYPE_OUTAGE
from ldap3 import AUTO_BIND_NO_TLS, Server, Connection, ALL

from ruxit.api.base_plugin import RemoteBasePlugin
from ruxit.api.exceptions import ConfigException

from ldap_imports.environment import get_api_url

DT_TIMEOUT_SECONDS = 10

@dataclass
class LDAPResult:
    response_time: float
    success: bool
    error_message: str
    content: str


class LDAPPlugin(RemoteBasePlugin):
    def initialize(self, **kwargs):
        self.executions = -1
        if not self.config.get("api_token"):
            raise ConfigException("API Token is required")
        if not self.config.get("test_name"):
            raise ConfigException("Test Name is required")

        if (
            not self.config.get("ldap_host")
            or not self.config.get("ldap_port")
            or not self.config.get("ldap_base_dn")
        ):
            raise ConfigException("'LDAP hostname or IP', 'LDAP Port' and 'LDAP Bind DN' are required")

    def query(self, **kwargs) -> None:
        self.executions += 1
        frequency = self.config.get("frequency", 1)
        if self.executions % frequency == 0:

            ldap_result = self.ldap_test()
            self.logger.info(f"LDAP test result: {ldap_result}")

            self.api_url = get_api_url()
            dt_client = Dynatrace(self.api_url, self.config["api_token"], timeout=DT_TIMEOUT_SECONDS)

            location_id = self.config.get("test_location", "Activegate") or "Activegate"
            test_id = f"{self.activation.entity_id}.{location_id}".lower()

            try:
                post_test_result = dt_client.third_part_synthetic_tests.report_simple_thirdparty_synthetic_test(
                    engine_name="LDAP",
                    timestamp=datetime.now(),
                    location_id=location_id,
                    location_name=location_id,
                    test_id=test_id,
                    test_title=self.config["test_name"],
                    step_title=self.config["test_name"],
                    schedule_interval=frequency * 60,
                    success=ldap_result.success,
                    response_time=int(ldap_result.response_time),
                    edit_link=f"#settings/customextension;id={self.plugin_info.name}",
                )
                self.logger.info(f"Post test result: {post_test_result}")

                post_event_response = dt_client.third_part_synthetic_tests.report_simple_thirdparty_synthetic_test_event(
                    test_id=test_id,
                    name=f"LDAP search failed for {self.config['test_name']}",
                    location_id=location_id,
                    timestamp=datetime.now(),
                    state="open" if not ldap_result.success else "resolved",
                    event_type=SYNTHETIC_EVENT_TYPE_OUTAGE,
                    reason=f"LDAP Search returned error: {ldap_result.error_message}",
                    engine_name="LDAP",
                )
                self.logger.info(f"Post event response: {post_event_response}")

                match_content = self.config.get("match_content", "")
                if match_content:
                    found = match_content in f"{ldap_result.content}"
                    self.logger.info(f"Matching content: '{match_content}' against '{ldap_result.content}' = {found}")

                    post_event_response = dt_client.third_part_synthetic_tests.report_simple_thirdparty_synthetic_test_event(
                        test_id=test_id,
                        name=f"LDAP search content did not match failed for {self.config['test_name']}",
                        location_id=location_id,
                        timestamp=datetime.now(),
                        state="open" if not found else "resolved",
                        event_type=SYNTHETIC_EVENT_TYPE_OUTAGE,
                        reason=f"Content '{match_content}' was not found in LDAP search result: '{ldap_result.content}'",
                        engine_name="LDAP",
                    )
                    self.logger.info(f"Post match event response: {post_event_response}")
            except Exception as e:
                self.logger.error(f"Error reporting third party test results to {self.api_url}: '{e}'")

    def ldap_test(self) -> LDAPResult:
        start_time = datetime.now()
        server = Server(
            host=self.config["ldap_host"],
            port=self.config["ldap_port"],
            get_info=ALL,
            use_ssl=self.config["ssl"],
        )

        try:
            self.logger.info(f"Connecting to LDAP server {server}")
            conn = Connection(
                server,
                user=self.config.get("ldap_bind_dn", ""),
                password=self.config["ldap_password"],
                auto_bind=AUTO_BIND_NO_TLS,
            )

        except Exception as e:
            error_message = f"Failed to connect to LDAP server {server}: {e}"
            self.logger.error(error_message)
            response_time = (datetime.now() - start_time).total_seconds() * 1000
            return LDAPResult(response_time, False, error_message,  "")

        try:
            self.logger.info(
                f"Searching server={server} base_dn={self.config['ldap_base_dn']} "
                f"filter={self.config['ldap_filters']}, attributes={self.config['ldap_attributes']}"
            )
            conn.search(
                search_base=self.config["ldap_base_dn"],
                search_filter=self.config["ldap_filters"],
                attributes=self.config["ldap_attributes"],
            )
            self.logger.info(
                f"Search results: {conn.result}, found {len(conn.entries)} entries"
            )
            response_time = (datetime.now() - start_time).total_seconds() * 1000
            return LDAPResult(response_time, True, "", f"{conn.result}")

        except Exception as e:
            error_message = f"Failed to search LDAP server {server}: {e}"
            self.logger.error(error_message)
            response_time = (datetime.now() - start_time).total_seconds() * 1000
            return LDAPResult(response_time, False, error_message, "")


